/******************************************************************
 * File:        JenaTest.java
 * Created by:  Dave Reynolds
 * Created on:  10 Oct 2011
 *
 * (c) Copyright 2011, Epimorphics Limited
 * $Id:  $
 *****************************************************************/

package com.epimorphics.support;

import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.IOException;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileUtils;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.Syntax;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Resource;
import java.io.File;
// import com.ge.research.sadl.sadlserver.server.ISadlServer;
// import com.ge.research.sadl.sadlserver.server.SadlServerImpl;
import com.ge.research.sadl.sadlserver.cxf.provider.ISadlServiceProvider;
import com.ge.research.sadl.sadlserver.cxf.provider.SadlServiceProvider;

public class JenaTest implements BundleActivator {

    public static final String DATA_FILE = "/resources/data.rdf";

    @Override
    public void start(BundleContext context) throws Exception {
        System.out.println("Test bundle ...");
        Model m = ModelFactory.createDefaultModel();
        InputStream in = JenaTest.class.getResourceAsStream(DATA_FILE);
        m.read(in, null, FileUtils.langXML);
        System.out.println( m.size() > 0 ? "SUCCESS" : "No data");
		// ---
		// File df = new File("D:\\projects\\SADL\\SadlServerV2Standalone\\DataModels\\ShapesDemo\\ServicesConfig.owl");
		// List<Object[]> lo = getServicesFromConfigFile(df);
		// System.out.println( lo.size() > 0 ? "SUCCESS" : "No data");
		// ---
		// ISadlServer ss = new SadlServerImpl("D:\\projects\\SADL\\SadlServerV2Standalone\\DataModels");
		// System.out.println( ss != null ? "SUCCESS" : "No server");
		// ---
		// ISadlServiceProvider ss = new SadlServiceProvider();
		// ss.setKbaseRoot("D:\\projects\\SADL\\SadlServerV2Standalone\\DataModels");
		// System.out.println( ss != null ? "SUCCESS" : "No server");
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        System.out.println("Bundle stopped");
    }
	
	private List<Object[]> getServicesFromConfigFile(File file) throws FileNotFoundException {
		List<Object[]> contents = null;
		Model m = ModelFactory.createMemModelMaker().createFreshModel();
		System.out.println("Model created");
		FileInputStream is = new FileInputStream(file);
		m = m.read(is, null);
		System.out.println("Model read");
		QueryExecution qexec = null;		
		com.hp.hpl.jena.query.ResultSet results = null;		
		String query = "select ?url ?sn ?mn where {?x <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://com.ge.research.sadl/sadlserver/Services#KnowledgeBase> . " +
				"?x <http://com.ge.research.sadl/sadlserver/Services#entryPoint> ?sn . " +
				"?sn <http://com.ge.research.sadl/sadlserver/Services#modelName> ?mn . " +
				"OPTIONAL{?x <http://com.ge.research.sadl/sadlserver/Services#url> ?url} }";
		qexec = QueryExecutionFactory.create(QueryFactory.create(query, Syntax.syntaxARQ), m);
		System.out.println("QueryExecutionFactory created");
		results = qexec.execSelect();
		System.out.println("Query performed");
		if (results != null) {
			while (results.hasNext()) {
				QuerySolution soln = results.nextSolution();
				String url = null;
				String sn = null;
				String mn = null;
				RDFNode urlnode = soln.get("?url");
				RDFNode snnode = soln.get("?sn");
				RDFNode mnnode = soln.get("?mn");
				if (urlnode != null) {
					if (urlnode instanceof Literal) {
						url = ((Literal)urlnode).getString();
					}
				}
				if (snnode instanceof Resource) {
					sn = ((Resource)snnode).getLocalName();
				}
				if (mnnode instanceof Literal) {
					mn = ((Literal)mnnode).getString();
				}
				if (sn != null && mn != null) {
					if (url != null) {
						if (!url.startsWith("http:") && !url.startsWith("file:")) {
							// relative url
							String kbr = file.getAbsolutePath(); //getKbaseRoot();
							url = kbr + (kbr.endsWith("/") || url.startsWith("//") ? "" : "/") + url;
						}
					}
					else {
						try {
							url = "file:/" + file.getCanonicalPath();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					if (contents == null) {
						contents = new ArrayList<Object[]>();
					}
					String[] info = new String[2];
					info[0] = url;
					info[1] = mn;
					Object[] content = new Object[2];
					content[0] = sn;
					content[1] = info;
					contents.add(content);
				}
			}
		}
		System.out.println("Returning contents");
		return contents;
	}

}

